from Globals import InitializeClass
from Globals import Persistent, Acquisition
import Globals
import AccessControl
import OFS
from AccessControl import ClassSecurityInfo
from DocumentTemplate.DT_Util import html_quote
from DocumentTemplate.DT_Var import newline_to_br
from webdav.common import rfc1123_date
from webdav.Lockable import ResourceLockedError
from webdav.WriteLockInterface import WriteLockInterface
from OFS.Cache import Cacheable
from XWorkFlow import XWorkFlow

_marker = None

class XImagem ( XWorkFlow, Cacheable, OFS.Image.Image ):
    """Classe que define o XImagem."""

    friendly_name = 'Imagem'
    texto = None
    ordem_layout = 0
    align = None
    border = 8
    _cache_namespace_keys=()
    security = ClassSecurityInfo()
    meta_type = 'XImagem'

    security.declareProtected('Modificar Propriedades da XImagem', 'manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXImagemForm", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX', 'gadgetPropertiesForm')
    gadgetPropertiesForm = Globals.DTMLFile("dtml/editXImagemForm", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZPropertiesForm')
    manage_ZPropertiesForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetPropertiesForm')

    def manage_options(self, REQUEST=None):
      manage_options=((
          {'label':'Propriedades', 'action':'manage_PropertiesForm'},
          ) + OFS.Image.Image.manage_options)

      zcomix_manage_options=((
          {'label':'Propriedades', 'action':'manage_ZPropertiesForm'},
          ) + OFS.Image.Image.manage_options)

      if self.SecurityGetUser().has_permission('View management screens', self):
        return manage_options
      else:
        return zcomix_manage_options

    def __init__(self, id, title, file, content_type='', precondition=''):
        self.__name__=id
        self.title=title
        self.precondition=precondition
        data, size = self._read_data(file)
        content_type=self._get_content_type(file, data, id, content_type)
        self.update_data(data, content_type, size)

    security.declareProtected('View', 'render')
    def render(self, REQUEST=None):
        "Retorna o conteudo do objeto. Utilizado na Interface do Zope."
        if not self._cache_namespace_keys:
           data = self.ZCacheable_get(default=_marker)
           if data is not _marker:
                # Return cached results.
                return data

        if self.align is None:
                result = self.tag(hspace=self.border, vspace=self.border)
        else:
                result =  self.tag(align=self.align, hspace=self.border, vspace=self.border)
        if not self._cache_namespace_keys:
            self.ZCacheable_set(result)
        return result

    security.declareProtected('Modificar Propriedades da XImagem', 'manage_Properties')
    def manage_Properties(self, title='', texto='', estilo_css=None,  align=None, border=8, in_ZMI=1, REQUEST=None):
        "Modifica as Propriedades da XImagem"
        self.title = title
        self.texto = texto
        self.estilo_css = estilo_css
        self.align = align
        self.border = border
        self.ZCacheable_invalidate()

        if REQUEST is not None:
          if in_ZMI:
            return self.manage_PropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Imagem <strong>%s</strong> foi editada.' % self.id
                )
          else:
            return self.manage_ZPropertiesForm(REQUEST, management_view='Propriedades',
                manage_tabs_message='A Imagem <strong>%s</strong> foi editada.' % self.id
                )

InitializeClass(XImagem)

def manage_addXImagem(self, id, file, title='', precondition='', content_type='', align=None, border=8, REQUEST=None):
    """ Adiciona um novo objeto XImagem."""
    id=str(id)
    title=str(title)
    content_type=str(content_type)
    precondition=str(precondition)

    id, title = cookId(id, title, file)

    self=self.this()

    # First, we create the image without data:
    self._setObject(id, XImagem(id,title,'',content_type, precondition))

    # Now we "upload" the data.  By doing this in two steps, we
    # can use a database trick to make the upload more efficient.
    self._getOb(id).manage_upload(file)
    self._getOb(id).align = align
    self._getOb(id).border = border

    if content_type:
        self._getOb(id).content_type=content_type

    if REQUEST is not None:
        try:    url=self.DestinationURL()
        except: url=REQUEST['URL1']
        REQUEST.RESPONSE.redirect('%s/manage_main' % url)
    return id

manage_addXImagemForm = Globals.HTMLFile('dtml/addXImagemForm', globals())

def default_alinhamentos(self):
        "Define os alinhamentos padrao para uma XImagem"
        result = [('Direita','right'), ('Esquerda','left')]
        return result

cookId = OFS.Image.cookId