import Globals
from Globals import Persistent, Acquisition
import AccessControl
import OFS
from AccessControl import ClassSecurityInfo
from AccessControl.Role import RoleManager
from XLink import XLink
from XConteudo import XConteudo
from XSecao import XSecao
import Products
from OFS.Application import Application
from App.Management import Navigation
from App.Management import Tabs
from AccessControl import SecurityManagement
from AccessControl import getSecurityManager, Unauthorized
from XWorkFlow import XWorkFlow
import time
import DateTime
from ZPublisher.BeforeTraverse import registerBeforeTraverse, NameCaller
from DocumentTemplate.DT_HTML import HTML
from StructuredText.StructuredText import HTML as STXHTML

class XSite ( XWorkFlow, Acquisition.Implicit, Persistent, RoleManager,  OFS.Folder.Folder ):
    """Classe que define o XSite."""

    friendly_name = 'Site'
    responsavel = ""
    data_inicial = "1970/01/01"
    data_final = "2030/01/01"
    template_principal = None
    secao_principal = None
    catalog_principal = None
    email_responsavel_site = "webmaster@localhost"

    def __setstate__(self, state):
        XWorkFlow.__setstate__(self, state)
        if not hasattr(self, '_noticias_capa'):
            self._noticias_capa = []
        if not hasattr(self, 'email_responsavel_site'):
          self.email_responsavel_site = "webmaster@localhost"

        self._oldManage=Navigation.manage

    def all_meta_types(self):
        """ What can you put inside me? """
        f = lambda x: x['name'] in ('DTML Method', 'DTML Document', 'Image', 'Z SQL Method',  'Script (Python)',
                                                'Folder', 'File', 'Formulator Form', 'XLink', 'XParagrafo', 'XImagem', 'XPagina',
                                                'XPaginaLayout', 'XSecao', 'XNoticia', 'Z MySQL Database Connection', 'MySQL User Folder',
                                                'Set Access Rule', 'Session Data Container', 'Session Data Manager', 'Session Id Manager',
                                                'SiteRoot', 'Z Search Interface', 'ZCatalog', 'LDAPLoginAdapter', 'LDAPUserManager', 'XTemplate')
        return filter(f, Products.meta_types)

    _properties = (
        {'id':'title','type':'string','mode':'w'},
        {'id':'responsavel','type':'string','mode':'w'},
        {'id':'data_inicial','type':'date','mode':'w' },
        {'id':'data_final','type':'date','mode':'w' },
        {'id':'template_principal','type':'string','mode':'w' },
        {'id':'secao_principal','type':'string','mode':'w' },
        {'id':'catalog_principal','type':'string','mode':'w' },
        )

    Caminho = XConteudo.Caminho
    itensMenu = XSecao.itensMenu
    itensMenuEsquerdo = XSecao.itensMenuEsquerdo
    itensMenuDireito = XSecao.itensMenuDireito

    security = ClassSecurityInfo()

    def manage_options(self, REQUEST=None):
        manage_options=(OFS.Folder.Folder.manage_options[:1] +(
            {'label':'Propriedades', 'action':'manage_PropertiesForm'},
            {'label':'Menu', 'action':'manage_MenuForm'},
            {'label':'Visualizar', 'action':''},
            {'label':'Noticias da Capa', 'action':'manage_SiteNewsForm'},
            ) + OFS.Folder.Folder.manage_options[3:] )

        zcomix_manage_options=((
            {'label':'Contedo', 'action':'manage_ZWorkspaceForm'},
            {'label':'Propriedades', 'action':'manage_ZPropertiesForm'},
            {'label':'Menu', 'action':'manage_ZMenuForm'},
            {'label':'Visualizar', 'action':''},
            {'label':'Noticias da Capa', 'action':'manage_ZSiteNewsForm'},
            ))

        if self.SecurityGetUser().has_permission('View management screens', self):
          return manage_options
        else:
          return zcomix_manage_options

    meta_type = 'XSite'

    security.declareProtected('Modificar Propriedades do XSite','manage_PropertiesForm')
    manage_PropertiesForm = Globals.DTMLFile("dtml/editXSiteForm", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetPropertiesForm')
    gadgetPropertiesForm = Globals.DTMLFile("dtml/editXSiteForm", globals(), in_ZMI=0)

    security.declareProtected('Modificar Propriedades do XSite','manage_MenuForm')
    manage_MenuForm = Globals.DTMLFile("dtml/editXSiteMenuForm", globals(), in_ZMI=1)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetMenuForm')
    gadgetMenuForm = Globals.DTMLFile("dtml/editXSiteMenuForm", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZEstiloCSS')
    manage_ZEstiloCSS = Globals.DTMLFile("dtml/estilos", globals())

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetHeader')
    gadgetHeader = Globals.DTMLFile("dtml/gadgetHeader", globals())

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetSearch')
    gadgetSearch = Globals.DTMLFile("dtml/gadgetSearch", globals())

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetTabs')
    gadgetTabs = Globals.DTMLFile("dtml/gadgetTabs", globals())

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetContents')
    gadgetContents = Globals.DTMLFile("dtml/gadgetContents", globals())

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetAddObject')
    gadgetAddObject = Globals.DTMLFile("dtml/gadgetAddObject", globals())

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','gadgetSearchResults')
    gadgetSearchResults = Globals.DTMLFile("dtml/gadgetSearchResults", globals())

    security.declareProtected('Selecionar Noticias para a Capa do Site','manage_SiteNewsForm')
    manage_SiteNewsForm = Globals.DTMLFile("dtml/editNoticiasCapaForm", globals(), in_ZMI=1)

    security.declareProtected('View','gadgetImagesForm')
    gadgetImagesForm = Globals.DTMLFile("dtml/gadgetImages", globals(), in_ZMI=0)

    security.declareProtected('Selecionar Noticias para a Capa do Site','gadgetSiteNewsForm')
    gadgetSiteNewsForm = Globals.DTMLFile("dtml/editNoticiasCapaForm", globals(), in_ZMI=0)

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZWorkspaceForm')
    manage_ZWorkspaceForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetContents')

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZSearchResultsForm')
    manage_ZSearchResultsForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetSearchResults')

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZPropertiesForm')
    manage_ZPropertiesForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetPropertiesForm')

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage_ZMenuForm')
    manage_ZMenuForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetMenuForm')

    security.declareProtected('Selecionar Noticias para a Capa do Site','manage_ZSiteNewsForm')
    manage_ZSiteNewsForm = Globals.DTMLFile("dtml/ZTemplate", globals(), in_ZMI=0, z_conteudo='gadgetSiteNewsForm')

    def __init__(self, id, title):
        self.id = id
        self.title = title

    security.declareProtected('View', 'index_html')
    def index_html(self, REQUEST=None):
        "Visualizacao principal (pagina inicial) do XSite"
        if self.restrictedTraverse(REQUEST.PATH_INFO) == self:
          secao_principal = self.getSecaoPrincipal()
          if secao_principal is not None and getattr(self, secao_principal, None) is not None:
            url = self.absolute_url()
            url += '/' + secao_principal
            return REQUEST.RESPONSE.redirect(url)
          else:
            body = """*ATENCAO.*

                              Este site ainda nao possui uma secao principal.

                              Em caso de duvida, entre em contato com "<dtml-var email_responsavel_site>":mailto:<dtml-var email_responsavel_site>.
                        """
            r = STXHTML(HTML(body)(self, REQUEST), header=0)
            return r

    def SecurityGetUser(self):
      return (SecurityManagement.getSecurityManager().getUser())

    security.declareProtected('Utilizar Interface de Gerenciamento do ZCoMIX','manage')
    def manage(self, REQUEST=None, *args, **kw):
        """Custom Manage"""
        if self.SecurityGetUser().has_permission('View management screens', self):
          return self._oldManage(self, REQUEST)
        else:
          return apply(self.manage_ZWorkspaceForm, [REQUEST], REQUEST.form)

    security.declareProtected('View','menuItemsPrincipal')
    def menuItemsPrincipal(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu do XSite"
        return self.itensMenuSuperior(REQUEST=REQUEST)

    security.declareProtected('View','getNoticiasCapa')
    def getNoticiasCapa(self, REQUEST=None):
        """ Lista de Noticias selecionadas para Capa do Site """
        result = []
        for i in self._noticias_capa:
          noticia = self.restrictedTraverse(i, None)
          if noticia is not None:
            result.append(noticia)

        return result

    security.declareProtected('Access contents information','itensMenuSuperior')
    def itensMenuSuperior(self, REQUEST=None):
        "Lista de items a serem mostrados no Menu do XSite"

        if REQUEST is not None:
                for i in range(0, len(self.REQUEST.PARENTS)):
                        dir = self.REQUEST.PARENTS[i]
                        if dir.meta_type=='XSite':
                                site = dir
                                break
        else:
                site = self

        items = []
        for item in site.objectValues(['XSecao', 'XPagina','XLink']):
                if self.SecurityGetUser().has_permission('Access contents information', item) and item.mostrar_no_menu_superior:
                        items.append(item)

        if len(items)>1:
                if items[0].ordem_menu_superior == items[1].ordem_menu_superior:
                        items.sort(lambda x, y: cmp(x.title_or_id(), y.title_or_id()))
                else:
                        items.sort(lambda x, y: cmp(x.ordem_menu_superior, y.ordem_menu_superior))

        return items

    security.declareProtected('Modificar Propriedades do XSite','addmenuItemPrincipal')
    def addmenuItemPrincipal(self, id, title=None, url=None, REQUEST=None):
        "Adiciona um novo Item de Menu"

        for i in range(0, len(self.REQUEST.PARENTS)):
                dir = self.REQUEST.PARENTS[i]
                if dir.meta_type=='XSite':
                        site = dir
                        break

        item = XLink(id, title, url)
        site._setObject(id, item)

        return self.manage_editXSiteMenuForm(REQUEST, management_view = 'Menu',
                        manage_tabs_message='O Site <strong>%s</strong> foi editado.' % self.id)

    def __before_publishing_traverse__(self, request=None, *args, **kw):
        if hasattr(request, 'TraversalRequestNameStack'):
          stack = request['TraversalRequestNameStack']
        else:
          return

        try:
          if stack[0][:6]=='manage':
            return
        except:
          pass

        obj = None
        template = None

        path = request.path[:]
        path.reverse()
        try:
          obj = self.unrestrictedTraverse(path)
        except:
          pass

        try:
          template = obj.getTemplate(request)
        except:
          pass

        if template is None:
          return

        if (not template in stack) and (not template in path):
          request['TraversalRequestNameStack'].append(template)

    security.declareProtected('Access contents information','getTemplatePrincipal')
    def getTemplatePrincipal(self, REQUEST=None):
        if self.template_principal is None:
          template_principal = self.objectValues("XTemplate")
          try:
            template_principal = template_principal[0].getId()
            self.template_principal = template_principal
          except:
            template_principal = None
        else:
          template_principal = self.template_principal
        return template_principal

    security.declareProtected('Access contents information','getCatalogPrincipal')
    def getCatalogPrincipal(self, REQUEST=None):
        if self.catalog_principal is None:
          catalog_principal = self.objectValues("ZCatalog")
          try:
                catalog_principal = catalog_principal[0].getId()
                self.catalog_principal = catalog_principal
          except:
                catalog_principal = None
        else:
          catalog_principal = self.catalog_principal
        return catalog_principal

    security.declareProtected('Access contents information','getSecaoPrincipal')
    def getSecaoPrincipal(self, REQUEST=None):
        if self.secao_principal is None:
          secao_principal = self.objectValues("XSecao")
          try:
                secao_principal = secao_principal[0].getId()
                self.secao_principal = secao_principal
          except:
                secao_principal = None
        else:
          secao_principal = self.secao_principal
        return secao_principal

    security.declareProtected('Selecionar Noticias para a Capa do Site','manage_siteNews')
    def manage_siteNews(self, noticias_capa, in_ZMI=1, REQUEST=None, *args, **kw):
        """ Modifica a selecao de noticias da capa do site  """
        if noticias_capa is None:
            noticias_capa = []
        self._noticias_capa = noticias_capa

        if REQUEST is not None:
          if in_ZMI:
                return self.manage_SiteNewsForm(REQUEST, management_view = 'Noticias da Capa',
                        manage_tabs_message='As noticias da capa do site <strong>%s</strong> foram modificadas.' % self.id)
          else:
                return self.manage_ZSiteNewsForm(REQUEST, management_view = 'Noticias da Capa',
                        manage_tabs_message='As noticias da capa do site <strong>%s</strong> foram modificadas.' % self.id)

    security.declareProtected('Modificar Propriedades do XSite','manage_Menu')
    def manage_Menu(self, menu_items, in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica a ordem dos Menus do XSite"
        ordem = 0
        for i in menu_items:
                ob = getattr(self, i, None)
                if ob is not None:
                        ob.ordem_menu_superior = ordem
                ordem += 1

        if REQUEST is not None:
          if in_ZMI:
                return self.manage_MenuForm(REQUEST, management_view = 'Menu',
                        manage_tabs_message='O Site <strong>%s</strong> foi editado.' % self.id)
          else:
                return self.manage_ZMenuForm(REQUEST, management_view = 'Menu',
                        manage_tabs_message='O Site <strong>%s</strong> foi editado.' % self.id)



    security.declareProtected('Modificar Propriedades do XSite','manage_Properties')
    def manage_Properties(self, title, responsavel='admin', data_inicial='1970/01/01', data_final='2030/01/01', template_principal=None, secao_principal=None, catalog_principal=None, email_responsavel_site = 'webmaster@localhost', in_ZMI=1, REQUEST=None, *args, **kw):
        "Modifica as Propriedades do XSite"
        self.title = title
        self.responsavel = responsavel
        self.email_responsavel_site = email_responsavel_site
        try: self.data_inicial = DateTime.DateTime(data_inicial).ISO()
        except: self.data_inicial = '1970/01/01'
        try: self.data_final = DateTime.DateTime(data_final).ISO()
        except: self.data_final = '2030/01/01'
        if catalog_principal is None:
          catalog_principal = self.getCatalogPrincipal()
        self.catalog_principal = catalog_principal
        if template_principal is None:
          template_principal = self.getTemplatePrincipal()
        self.template_principal = template_principal
        if secao_principal is None:
          secao_principal = self.getSecaoPrincipal()
        self.secao_principal = secao_principal
        if REQUEST is not None:
          if in_ZMI==1:
                return self.manage_PropertiesForm(REQUEST, management_view = 'Propriedades',
                        manage_tabs_message='O Site <strong>%s</strong> foi editado.' % self.id)
          else:
                return self.manage_ZPropertiesForm(REQUEST, management_view = 'Propriedades',
                        manage_tabs_message='O Site <strong>%s</strong> foi editado.' % self.id)


Globals.default__class_init__(XSite)


def manage_addXSite(self, id, title, responsavel='admin', data_inicial='1970/01/01', data_final='2030/01/01', template_principal=None, secao_principal=None, catalog_principal=None, email_responsavel_site = 'webmaster@localhost', REQUEST=None):
    """Cria um XSite e o adiciona ao seu parent Folder.
    O argumento 'self' referencia o parent Folder.
    """
    site = XSite(id, title)
    site.responsavel = responsavel
    try: site.data_inicial = DateTime.DateTime(data_inicial).ISO()
    except: site.data_inicial = '1970/01/01'
    try: site.data_final = DateTime.DateTime(data_final).ISO()
    except: site.data_final = '2030/01/01'
    site.catalog_principal = catalog_principal
    site.template_principal = template_principal
    site.secao_principal = secao_principal
    site.email_responsavel_site = email_responsavel_site
    self._setObject(id, site)
    site = getattr(self, id)
    registerBeforeTraverse(site, NameCaller('lambda: return'), 'Access Rule')
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

manage_addXSiteForm = Globals.HTMLFile('dtml/addXSiteForm', globals())

