import Globals, ExtensionClass
from Globals import InitializeClass, Persistent
from AccessControl import ClassSecurityInfo
from AccessControl import SecurityManagement
import time
import DateTime
from DocumentTemplate.DT_Var import url_quote_plus
from OFS import Application

class XWorkFlow ( ExtensionClass.Base ):
    """Classe que define o XLink."""

    meta_type = 'XWorkFlow'
    produtor = ""
    data_producao = "1970/01/01"
    revisor = ""
    data_revisao = "1970/01/01"
    publicador = ""
    data_publicacao = "1970/01/01"

    responsavel = ""
    data_inicial = "1970/01/01"
    data_final = "2030/01/01"
    _valores_status = {}
    _valores_status['Produzindo'] = 'produzindo'
    _valores_status['Produzido'] = 'produzido'
    _valores_status['Revisando'] = 'revisando'
    _valores_status['Revisado'] = 'revisado'
    _valores_status['Publicado'] = 'publicado'

    security = ClassSecurityInfo()

    def __setstate__(self, state):
        Persistent.__setstate__(self, state)
        if hasattr(self, 'data_final') and self.data_final == '2100/01/01':
           self.data_final = '2030/01/01'
        if hasattr(self, 'data_producao') and self.data_producao == '1970/01/01':
            t = time.time()
            self.data_producao = DateTime.DateTime(t).ISO()
        if hasattr(self, 'data_revisao') and self.data_revisao == '1970/01/01':
            t = time.time()
            self.data_revisao = DateTime.DateTime(t).ISO()
        if hasattr(self, 'data_publicacao') and self.data_publicacao == '1970/01/01':
            t = time.time()
            self.data_publicacao = DateTime.DateTime(t).ISO()
        if not hasattr(self, '_status'):
          self._status = self._valores_status['Publicado']
        if hasattr(self, 'produtor') and self.produtor=="":
          self.produtor = 'admin'
        if hasattr(self, 'revisor') and self.revisor=="":
          self.revisor = 'admin'
        if hasattr(self, 'publicador') and self.publicador=="":
          self.publicador = 'admin'

    security.declareProtected('Access Content Information', 'getStatus')
    def getStatus(self, REQUEST=None):
      """ Retorna o status do conteudo """
      if hasattr(self, '_status'):
        return self._status
      else:
        return 'publicado'

    security.declareProtected('Access Content Information', 'getStatusName')
    def getStatusName(self, REQUEST=None):
      """ Retorna o titulo do status do conteudo """
      for i in self._valores_status.keys():
        if self._valores_status[i] == self._status:
          return i

    security.declareProtected('Produzir Conteudo', 'manage_statusProduzido')
    def manage_statusProduzido(self, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Produzido """
      self._status = self._valores_status['Produzido']
      self.produtor = REQUEST.AUTHENTICATED_USER.getId()
      t = time.time()
      self.data_producao = DateTime.DateTime(t).ISO()
      if REQUEST is not None:
        if REQUEST.has_key('in_ZMI') and REQUEST['in_ZMI']=="0":
          return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % self._status) )
        else:
          return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % self._status) )

    security.declareProtected('Produzir Conteudo', 'manage_statusProduzindo')
    def manage_statusProduzindo(self, REQUEST=None, id_original=None):
      """ Modifica o status do conteudo para o estado de Produzindo """
      ob = self
      if id_original is not None:
        id_novo = id_original + '_' +  str(int(time.time()))
        self.aq_parent.manage_clone(self, id_novo)
        ob = getattr(self.aq_parent, id_novo)
        ob._id_original = id_original
      ob._status = ob._valores_status['Produzindo']
      ob.produtor = REQUEST.AUTHENTICATED_USER.getId()
      t = time.time()
      ob.data_producao = DateTime.DateTime(t).ISO()
      if REQUEST is not None:
        if REQUEST.has_key('in_ZMI') and REQUEST['in_ZMI']=="0":
          return REQUEST.RESPONSE.redirect(ob.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % ob._status) )
        else:
          return REQUEST.RESPONSE.redirect(ob.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % ob._status) )

    security.declareProtected('Revisar Conteudo', 'manage_statusRevisado')
    def manage_statusRevisado(self, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Revisado """
      self._status = self._valores_status['Revisado']
      self.revisor = REQUEST.AUTHENTICATED_USER.getId()
      t = time.time()
      self.data_revisao = DateTime.DateTime(t).ISO()
      if REQUEST is not None:
        if REQUEST.has_key('in_ZMI') and REQUEST['in_ZMI']=="0":
          return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % self._status) )
        else:
          return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % self._status) )

    security.declareProtected('Revisar Conteudo', 'manage_statusRevisando')
    def manage_statusRevisando(self, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Revisando """
      self._status = self._valores_status['Revisando']
      self.revisor = REQUEST.AUTHENTICATED_USER.getId()
      t = time.time()
      self.data_revisao = DateTime.DateTime(t).ISO()
      if REQUEST is not None:
        if REQUEST.has_key('in_ZMI') and REQUEST['in_ZMI']=="0":
          return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % self._status) )
        else:
          return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % self._status) )

    security.declareProtected('Publicar Conteudo', 'manage_statusPublicado')
    def manage_statusPublicado(self, REQUEST=None):
      """ Modifica o status do conteudo para o estado de Publicado """
      self._status = self._valores_status['Publicado']
      self.publicador = REQUEST.AUTHENTICATED_USER.getId()
      t = time.time()
      self.data_publicacao = DateTime.DateTime(t).ISO()
      if hasattr(self, '_id_original'):
        self.aq_parent.manage_delObjects(ids=self._id_original)
        self.aq_parent.manage_renameObject(id=self.getId(), new_id=self._id_original)
        del self._id_original
      if REQUEST is not None:
        if REQUEST.has_key('in_ZMI') and REQUEST['in_ZMI']=="0":
          return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % self._status) )
        else:
          return REQUEST.RESPONSE.redirect(self.absolute_url() + '/manage_workspace?management_view=Propriedades&manage_tabs_message='+url_quote_plus('Status modificado para %s.' % self._status) )

    def checkPublished(self, REQUEST=None, *args, **kw):
      """ Checa pela data inicial e final e o status de publicado """
      if hasattr(self, 'data_inicial') and hasattr(self, 'data_final'):
        if (DateTime.DateTime(self.data_inicial).ISO() < DateTime.DateTime().ISO()) \
        and (DateTime.DateTime(self.data_final).ISO() > DateTime.DateTime().ISO()) \
        and self.getStatus()=='publicado':
          return 1
        return 0

"""
    def __before_publishing_traverse__(self, REQUEST=None, *args, **kw):
      sm = SecurityManagement.getSecurityManager()
      if hasattr(self, 'data_inicial') and hasattr(self, 'data_final'):
        if (DateTime.DateTime(self.data_inicial).ISO() < DateTime.DateTime().ISO()) \
        and (DateTime.DateTime(self.data_final).ISO() > DateTime.DateTime().ISO()) \
        and self.getStatus()=='publicado':
          if not (sm.checkPermission('View', self) \
          and sm.checkPermission('Access contents information', self)):
            self.manage_permission(permission_to_manage='View', roles=[], acquire=1, REQUEST=None)
            self.manage_permission(permission_to_manage='Access contents information', roles=[], acquire=1, REQUEST=None)
        else:
          if (sm.checkPermission('View', self) \
          and sm.checkPermission('Access contents information', self)):
            self.manage_permission(permission_to_manage='View', roles=['Manager'], acquire=0, REQUEST=None)
            self.manage_permission(permission_to_manage='Access contents information', roles=['Manager'], acquire=0, REQUEST=None)
"""

InitializeClass(XWorkFlow)

